package com.zeroturnaround.javarebel.aspectj;

import org.aspectj.weaver.loadtime.IWeavingContext;

import java.lang.reflect.Field;
import org.aspectj.weaver.loadtime.Aj;
import org.aspectj.weaver.loadtime.ClassPreProcessor;
import org.aspectj.weaver.loadtime.ClassPreProcessorAgentAdapter;
import org.zeroturnaround.javarebel.ClassBytecodeProcessor;
import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.IntegrationFactory;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.Plugin;

public class AspectJPlugin implements Plugin {

  ThreadLocal circ = new ThreadLocal();

  public void preinit() {
    try {
      // Get s_preProcessor field
      Field  s_preProcessorField = ClassPreProcessorAgentAdapter.class.getDeclaredField("s_preProcessor");
      s_preProcessorField.setAccessible(true);
      
      IWeavingContext oldIWeavingContext = null;
      
      // Get weavingContext field value
      Aj oldAj = (Aj) s_preProcessorField.get(null);
      if (oldAj != null) {
        Field weavingContextField = Aj.class.getDeclaredField("weavingContext");
        weavingContextField.setAccessible(true);
        oldIWeavingContext = (IWeavingContext) weavingContextField.get(oldAj);
      }
      
      final IWeavingContext weavingContext = oldIWeavingContext;

      // Set new ClassPreProcessor lazily intializing the new Aj
      s_preProcessorField.set(null, new ClassPreProcessor() {
        volatile boolean registered = false;
        
        public byte[] preProcess(String arg0, byte[] arg1, ClassLoader arg2) {
          if (!registered) {
            registered = true;
            
            final Aj aj = new Aj(weavingContext);
            aj.initialize();

            IntegrationFactory.getInstance().addIntegrationProcessor(new ClassBytecodeProcessor() {
              public byte[] process(ClassLoader cl, String classname, byte[] bytecode) {
                if (circ.get() != null)
                  return bytecode; 
                
                try {
                  circ.set(Boolean.TRUE);
                  return aj.preProcess(classname, bytecode, cl);
                }
                finally {
                  circ.set(null);
                }
              }
            }, false);
          }
          return arg1;
        }

        public void initialize() {
        }
      });
      
    } catch (Throwable e) {
      LoggerFactory.getInstance().errorEcho(e);      
    }
  }

  public boolean checkDependencies(ClassLoader cl, ClassResourceSource crs) {
    return crs.getClassResource("org.aspectj.weaver.loadtime.ClassPreProcessorAgentAdapter") != null;
  }

  public String getId() {
    return "aspectj_plugin";
  }

  public String getName() {
    return "AspectJ Plugin";
  }

  public String getDescription() {
    return "Allows the AspectJ load-time weaving (javaagent launched by aj5 or -javaagent:aspectjweaver.jar) to be used with JavaRebel. " +
    		"Note that AspectJ weaver will still show \"java.lang.Exception: AspectJ5 does not weave hotswapped class\" in the " +
    		"beginning, but this can be safely ignored.";
  }

  public String getAuthor() {
    return null;
  }

  public String getWebsite() {
    return null;
  }
}
